//+------------------------------------------------------------------+
//|                                                          MTP.mq4 |
//|                               Copyright  2006, Taylor Stockwell |
//|                                             Version: 1.10_unoff  |
//|                      Unoff version: modif by rafo05@hotmail.com  |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, Taylor Stockwell"
#property link      "stockwet@yahoo.com"

#include <stderror.mqh>
#include <stdlib.mqh>

/*
Updates:

   11/2/2006:: Fixed bug that caused the sl var to be set to 1 when a 
               pending order was > than the First_Stop_Target value.
   1/23/2007:: Fixed pip calculation. The old way of calculating pips 
               could be off by 1 pip, resulting in missed TP or move 
               stops. The new calculation is more accurate.
   5/31/2007:: Lousan:  Added a Third Stop Loss.  Renamed EA to MTP 
               to differentiate it from Stockwell's version.
   6/01/2007:: Lousan:  Added a pip by pip trailing stop loss to kick 
               in after the last fixed Stop Loss is in place.
   7/27/2007:: Lousan:  Ver 1.2  Added Version # and Revision date and 
               Pre-Set Name to display.   
   8/09/2007:: Lousan:  Ver 1.3  Trailing stop was not being moved when
               fixed stops were zero.
   8/14/2007:: Lousan:  Cosmetic change to Legend
   8/17/2007:: Lousan:  Ver 1.31  Added email and sound alerts
   8/24/2007:: Lousan:  Ver 1.4  Revised to make the Trailing Stop an 
               imaginary stop, ie. don't send a Move Stop request to the 
               server, but keep track of the stop within MTP, like it 
               does with Max Loss.
   8/27/2007:: Lousan:  Ver 1.5 If fixed Stop 2 or 3 was zero, then 
               previous current stop would be set to zero.  Fixed that.
   9/03/2007:: Lousan:  Ver 1.51  Revised to set Max_Loss to 999 if 
               input value is zero.  Also added display for open and close 
               (last) spreads and for Max Pips (high water mark).
   9/21/2007:: Lousan:  Ver. 1.6  Added option to choose of closing lots.  
               Method 1 closes one position at each TP point.  Method 2  
               increases number of lots to close at each TP point by 
               adding the original Lots to Close to the current number.
   9/26/2007:: Lousan:  Ver. 1.7  Increased the number of TP targets from 
               1 to 3.  Removed the Move_Stops and Use_Max_Loss input 
               parameters.            
   3/19/2008:: SwingMan:  Ver. 1.8 In the Comment() row with:
               ""\nOrder Open: ",dOrderOpenPrice," the use of OrderOpenPrice() was false.
   Dec/15/2012: by rafo05@hotmail. Ver. 1.10_unoff
               "First_Stop_Target", "Second_Stop_Target" and "Third_Stop_Target" renamed as 
                  "First_StopLoss_Trigger", "Second_StopLoss_Trigger" and "Third_StopLoss_Trigger".
               Option to define specific ticket number to apply EA.
               Include SLIP as slippage definable variable.
               Include drawing of TP lines on Init() and SL line when moving Stop Loss level.
               ErrorDescription function added.
               "... used the Point variable to convert pips to points in my MQL4 programs.
                  For example, if I want to calculate a stop loss value for a buy trade, I would multiply
                  the required number of pips by Point and subtract the result from the Ask price" (http://www.metatrader4.com/forum/12624)
               
*/

//=============== VARS external
extern string PreSet_Name = "Default";
extern double Slip = 3;          //Pre-defined slippage for open position
extern int First_TP = 100;
extern int Second_TP = 200;
extern int Third_TP = 0;    //initialy 300
extern int TP_Increment = 100;
extern double Close_Lots = 0.01;     //initially 0.1
extern int Close_Method = 1;
extern int First_StopLoss_Trigger = 50;
extern int First_Stop = 25;
extern int Second_StopLoss_Trigger = 100;
extern int Second_Stop = 50;
extern int Third_StopLoss_Trigger = 0;    //initially 200
extern int Third_Stop = 0;           //initially 100
extern int Trail_Stop_By = 15;      //initially 100
extern int Max_Loss = 300;
extern bool Alert_Sound = true;
extern bool Alert_Email = false;
extern int Magic_Number = 0;
extern int instanceID = 5;    //don't know yet how to use it. IBFX uses it to avoid lost EA progress if it is restarted
extern int TicketNumber = 0;  //variable to be used if specific ticket number is defined. If "0" means this option is not used

//=============== VARS internal
int nextTP;
int sl;
int range = 5;
int multiplier;
int Trail_Stop;
int Trail_Trigger;
int Open_Spread=0;
int Close_Spread=0;
int Max_Pips=-99;
int temp;
double Lots2Close;
string trailon;
string version="MTP 1.10  Dec. 15, 2012   ";
double dOrderOpenPrice;
double TP3_price;
double TP2_price;
double TP1_price;
// OrderType == 1 is OP_SELL

// line properties to be drawn
int      linestyle=STYLE_DOT;
int      linewidth=2;
color    TPlinecolor=Lime;     // TP line properties to be drawn
color    SLlinecolor=Red;      // SL line properties to be drawn
color    SL_oldlinecolor=Brown;   // old SL line properties to be drawn when SL is moved


//=============== FUNCTION init
int init()
  {
   // Don't know why these initializations are needed because all this code in included on START function
   
   int tOrders = OrdersTotal();     //Returns open and pending orders count shown on Terminal pannel. Intention is to check whether there are open orders or not
   if(First_Stop==0 && First_StopLoss_Trigger !=0) First_Stop=1;
   if(Max_Loss < 0) Max_Loss=(-1*Max_Loss);  //changed into positive number if input value is negative
   if(Max_Loss==0)  Max_Loss=999;   //v1.51: set Max_Loss to 999 if input value is zero.
   if(Trail_Stop_By < 0) Trail_Stop_By = -1*Trail_Stop_By;  //changed into positive number if input value is negative
   if( tOrders == 0){  sl = -1*Max_Loss;  //??
 

 
   if(First_TP!=0) nextTP = First_TP;  //verify if 1st_TP exist, if so, nextTP = First_TP
   else nextTP = TP_Increment;         //verify if 1st_TP exist, if not TP_increment will be used
   
   Trail_Stop = sl;
   Print("f_init: Trail_Stop = sl ",sl,".");  //debug info check: don't know when sl was initialized to assign value to Trail_Stop!
   trailon = "Off ";}
   getMaxLoss();    // ???? Why is getMaxLoss function necessary? ~Lousan
   if(Third_Stop!=0) {Trail_Trigger=Third_Stop+Trail_Stop_By;   //if 3rd TP exist, Trail_Trigger is assigned with value of 3rd TP + Trail_Stop_By
                                 //to trail SL after the last targeted SL, or from the Max_Loss if you're not using targeted stops
                                 Print("f_init: Trail_Trigger=Third_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                 Print("f_init: Third_Stop: ",Third_Stop,".");  //debug info check
                                 Print("f_init: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
   }
   else if(Second_Stop!=0) {Trail_Trigger=Second_Stop+Trail_Stop_By; // if enters here mend there is no 3rd TP (Third_Stop=0),
                                 // so Trail_Trigger is assigned with value of 2nd TP + Trail_Stop_By
                                 Print("f_init: Trail_Trigger=Second_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                 Print("f_init: Second_Stop: ",Second_Stop,".");  //debug info check
                                 Print("f_init: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                 }
   else if(First_Stop!=0) {Trail_Trigger=First_Stop+Trail_Stop_By; // if enters here mend there is no 2nd TP (Second_Stop=0),
                                 // so Trail_Trigger is assigned with value 1st TP + Trail_Stop_By
                                 Print("f_init: Trail_Trigger=Second_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                 Print("f_init: First_Stop: ",First_Stop,".");  //debug info check
                                 Print("f_init: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                 }
   else {Trail_Trigger=sl+Trail_Stop_By;    // if enters here mend there is no target TP defined,
                                 //so Trail_Trigger is assigned with value sl + Trail_Stop_By
                                 // again.. don't know when sl was initialized!!!
                                 Print("f_init: Trail_Trigger=sl+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                 Print("f_init: sl: ",sl,".");  //debug info check
                                 Print("f_init: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                 }
   Lots2Close = Close_Lots;
   
   Comment(version, 
   "\nPreSet . . . ", PreSet_Name,
   "\nClose Method:  ", Close_Method,  
   "\nClose Lots:    ", Close_Lots,
   "\nTP:   ", First_TP, "  ", Second_TP, "  ",
               Third_TP, "  +", TP_Increment,
   "\nSL1:  ", First_StopLoss_Trigger, "  ", First_Stop,
   "\nSL2:  ", Second_StopLoss_Trigger, "  ", Second_Stop,
   "\nSL3:  ", Third_StopLoss_Trigger,  "  ", Third_Stop,
   "\nTSL:  ", Trail_Stop_By,
   "\nMax Loss:  ", Max_Loss);
  }
//== end function


//=============== FUNCTION deinit
int deinit()
  {
   int tOrders = OrdersTotal();     //Returns open and pending orders count shown on Terminal pannel.
                                    //Intention is to check whether there are open orders or not
                                    
   if( tOrders == 0)                //IF "tOrders == 0"... (means there are no orders on Terminal pannel)
   {
      sl = -1*Max_Loss;
      nextTP = First_TP;
      Trail_Stop = sl;
      Comment("");      //remove comments on screen
   }
   ObjectsDeleteAll();     //delete all lines drawn
   return(0);
  }
//== end function

//========== FUNCTION Start
int start()
  {
   int tOrders = OrdersTotal();  //Returns open and pending orders count shown on Terminal pannel.
                                 //Intention is to check whether there are open orders or not
                                 
   if( tOrders == 0)             //IF "tOrders == 0"... (means there no orders on Terminal pannel)
   {
      Print("f_start: tOrders=0. Enter on reinicialization IF clause");  //debug info check
      if(First_Stop==0 && First_StopLoss_Trigger !=0) First_Stop=1; //define minimal SL, because 1st SL not defined but 1st SL Trigger defined
      if(Max_Loss < 0) Max_Loss=(-1*Max_Loss);  //change it into positive number if it was defined as negative
      if(Max_Loss==0)  Max_Loss=999;            //set Max_Loss to 999 if  input value is zero
      if(Trail_Stop_By < 0) Trail_Stop_By = -1*Trail_Stop_By;  //change it into positive number if it was defined as negative
      sl = -1*Max_Loss;          //inizialising SL as negative value of Max_Loss defined (Why??????)
      Print("f_start: sl = -1*Max_Loss: ",sl,".");  //debug info check
      
      if(First_TP!=0) nextTP = First_TP;  //because there is a 1st TP planned, value is assigned to variable nextTP
      else nextTP = TP_Increment;         // because there is no 1st TP planned, value of TP_increment is assigned to variable nextTP
      
      Trail_Stop = sl;  //Trail_Stop is assiged to "sl = -1*Max_Loss" value
      trailon = "Off ";
      getMaxLoss();    // ???? Why is getMaxLoss function necessary? ~Lousan
      
      if(Third_Stop!=0) {Trail_Trigger=Third_Stop+Trail_Stop_By;   //if there 3rd TP exist, Trail_Trigger is assigned with value of 3rd TP + Trail_Stop_By
                                   //to trail SL after the last targeted SL, or from the Max_Loss if you're not using targeted stops
                                   Print("f_start: Trail_Trigger=Third_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                   Print("f_start: Third_Stop: ",Third_Stop,".");  //debug info check
                                   Print("f_start: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                   if (OrderType() == OP_SELL) TP3_price = NormalizeDouble (OrderOpenPrice() - Third_TP*Point, Digits);
                                   if (OrderType() == OP_BUY) TP3_price = NormalizeDouble (Third_TP*Point - OrderOpenPrice(), Digits);
                                   Print("f_start: TP3_price: ",TP3_price,".");  //debug info check
                                   }
      else if(Second_Stop!=0) {Trail_Trigger=Second_Stop+Trail_Stop_By; // if enters here mend there is no 3rd TP (Third_Stop=0),
                                   // so Trail_Trigger is assigned with value of 2nd TP + Trail_Stop_By
                                   Print("f_start: Trail_Trigger=Second_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                   Print("f_start: Second_Stop: ",Second_Stop,".");  //debug info check
                                   Print("f_start: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                   if (OrderType() == OP_SELL) TP2_price = NormalizeDouble (OrderOpenPrice() - Second_TP*Point, Digits);
                                   if (OrderType() == OP_BUY) TP2_price = NormalizeDouble (Second_TP*Point - OrderOpenPrice(), Digits);
                                   Print("f_start: TP2_price: ",TP2_price,".");  //debug info check
                                   }
      else if(First_Stop!=0) {Trail_Trigger=First_Stop+Trail_Stop_By; // if enters here mend there is no 2nd TP (Second_Stop=0),
                                   // so Trail_Trigger is assigned with value 1st TP + Trail_Stop_By
                                   Print("f_start: Trail_Trigger=Second_Stop+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                   Print("f_start: First_Stop: ",First_Stop,".");  //debug info check
                                   Print("f_start: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                   if (OrderType() == OP_SELL) TP1_price = NormalizeDouble (OrderOpenPrice() - First_TP*Point, Digits);
                                   if (OrderType() == OP_BUY) TP1_price = NormalizeDouble (First_TP*Point - OrderOpenPrice(), Digits);
                                   Print("f_start: TP1_price: ",TP1_price,".");  //debug info check
                                   }
      else {Trail_Trigger=sl+Trail_Stop_By;    // if enters here mend there is no target TP defined,
                                   //so Trail_Trigger is assigned with value sl + Trail_Stop_By
                                   // again.. don't know when sl was initialized!!!
                                   Print("f_start: Trail_Trigger=sl+Trail_Stop_By: ",Trail_Trigger,".");  //debug info check
                                   Print("f_start: sl: ",sl,".");  //debug info check
                                   Print("f_start: Trail_Stop_By: ",Trail_Stop_By,".");  //debug info check
                                   }
      Lots2Close = Close_Lots;
   }
   getOpenOrders();  //Either there are orders on terminal pannel or not, this will be executed... why???
   getSpread();
   //Comment(sl);

   return(0);
  }
//== end function Start

//========== FUNCTION getSpread
int getSpread()
{
   int spread=MarketInfo(Symbol(),MODE_SPREAD);
   if(Open_Spread==0) Open_Spread=spread;
   Close_Spread=spread;
   Print("f_getSpread: Open_Spread=spread ",Open_Spread,".");  //debug info check
   Print("f_getSpread: Close_Spread=spread ",Close_Spread,".");  //debug info check
   Print("f_getSpread: returning spread= ",spread,".");  //debug info check
   return(spread);
}
//== end function getSpread

//========== FUNCTION getMaxLoss
int getMaxLoss()
{
   int calcMaxLoss;
   calcMaxLoss = Max_Loss;
   return(calcMaxLoss);
}
//== end function getMaxLoss

//========== FUNCTION getOpenOrders
void getOpenOrders()
{
   int nsl, nsd;
   string mngMagic;
   
   //if (TicketNumber != 0)   //because an specific ticket number has been defined
   //{
   // OrderSelect(TicketNumber, SELECT_BY_TICKET);
   //}
   //else
   //{
   int totalorders = OrdersTotal();    //Returns open and pending orders count shown on Terminal pannel.
   Print("f_getOpenOrders: OrdersTotal()= ",totalorders,".");  //debug info check
   for(int j=0; j<totalorders;j++)     //analysis will be done for all open and pending orders shown on Terminal pannel
                                       //looping for 0, 1, 2... up to amount of count of orders shown on Terminal pannel
   {
      OrderSelect(j, SELECT_BY_POS, MODE_TRADES);  //MODE_TRADES (default)- order selected from trading pool(opened and pending orders)
      if((OrderType() == OP_BUY || OrderType() == OP_SELL) &&  //Either order is BUY or SELL (what ever it is)
        OrderSymbol() == Symbol()&&                            // AND currency pair of order is the same as on chart EA is attached
        (Magic_Number==0 || Magic_Number == OrderMagicNumber()))  //AND either ther is Magic_Number defined on order or ther is not at all (not working with Magic Number
                                                                  //if so, THEN...
      {
         dOrderOpenPrice = OrderOpenPrice();    //get open price of analized order
         Print("f_getOpenOrders: OrderOpenPrice()= ",dOrderOpenPrice,".");  //debug info check
         Print("f_getOpenOrders: OrderType()= ",OrderType(),".");  //debug info check
         Print("f_getOpenOrders: NormalizeDouble( OrderLots(), 1 )= ",NormalizeDouble( OrderLots(), 1 ),".");  //debug info check
         Print("f_getOpenOrders: OrderLots()= ",OrderLots(),".");  //debug info check
         Print("f_getOpenOrders: Digits= ",Digits,".");  //debug info check
         Print("f_getOpenOrders: Point= ",Point,".");  //debug info check
         Print("f_getOpenOrders: NormalizeDouble( OrderOpenPrice(), Digits)= ",NormalizeDouble( OrderOpenPrice(), Digits),".");  //debug info check
         Print("f_getOpenOrders: OrderOpenPrice()= ",OrderOpenPrice(),".");  //debug info check
         Print("f_getOpenOrders: NormalizeDouble( OrderStopLoss(), Digits)= ",NormalizeDouble( OrderStopLoss(), Digits),".");  //debug info check
         Print("f_getOpenOrders: OrderStopLoss()= ",OrderStopLoss(),".");  //debug info check
         Print("f_getOpenOrders: NormalizeDouble( OrderTakeProfit(), Digits)= ",NormalizeDouble( OrderTakeProfit(), Digits),".");  //debug info check
         Print("f_getOpenOrders: OrderTakeProfit()= ",OrderTakeProfit(),".");  //debug info check
         double val=getPipValue(OrderOpenPrice(),OrderType()); //Calculate amount of PIPs of current price vs open price
         Print("f_getOpenOrders: val=getPipValue ",val,".");  //debug info check
         draw_horizontal_TP_line();        //added function to draw TP lines
         checkStops(val,OrderTicket());
         if(val > nextTP) takeProfit(val,OrderTicket());
      }
      
      if(Magic_Number == 0)
         mngMagic = "All "+Symbol()+" trades.";
      else
         mngMagic = "Trades with magic number = "+Magic_Number;


      if(sl==-1*Max_Loss)
      {
         Print("f_getOpenOrders: if(sl==-1*Max_Loss)= ",sl,".");  //debug info check
         nsl = First_StopLoss_Trigger;
         nsd = First_Stop;
         Print("f_getOpenOrders: nsl= ",nsl,".");  //debug info check
         Print("f_getOpenOrders: nsd= ",nsd,".");  //debug info check
      }
      else if(sl==First_Stop)
      {
         Print("f_getOpenOrders: if(sl==First_Stop)= ",sl,".");  //debug info check
         nsl = Second_StopLoss_Trigger;
         nsd = Second_Stop;
         Print("f_getOpenOrders: nsl= ",nsl,".");  //debug info check
         Print("f_getOpenOrders: nsd= ",nsd,".");  //debug info check
      } 
      else if(sl==Second_Stop)
      {
         Print("f_getOpenOrders: if(sl==Second_Stop)= ",sl,".");  //debug info check
         nsl = Third_StopLoss_Trigger;
         nsd = Third_Stop;
         Print("f_getOpenOrders: nsl= ",nsl,".");  //debug info check
         Print("f_getOpenOrders: nsd= ",nsd,".");  //debug info check
      }
      else if(Trail_Stop_By != 0 && val >= Trail_Trigger)
      {
         Print("f_getOpenOrders: if(Trail_Stop_By != 0 && val >= Trail_Trigger)= ",sl,".");  //debug info check
         nsl = val + 1;
         nsd = val + 1 - Trail_Stop_By; 
         trailon = " On";
         Print("f_getOpenOrders: nsl= ",nsl,".");  //debug info check
         Print("f_getOpenOrders: nsd= ",nsd,".");  //debug info check
      }
      //RefreshRates();
      Comment(version, 
      "\nPreSet . . . ", PreSet_Name,
      "\nClose Method:  ", Close_Method,
      "\nClose Lots:    ", Close_Lots,
      "\nTP:   ", First_TP, "  ", Second_TP, "  ",
                  Third_TP, "  +", TP_Increment,
      "\nSL1:  ", First_StopLoss_Trigger, "  ", First_Stop,
      "\nSL2:  ", Second_StopLoss_Trigger, "  ", Second_Stop,
      "\nSL3:  ", Third_StopLoss_Trigger,  "  ", Third_Stop,
      "\nTSL:  ", Trail_Stop_By,
      "\nMax Loss:  ", Max_Loss,
      "\n----------------------------",
      "\nOrder Open: ",dOrderOpenPrice,         
      "\nOpen Spread:  ",Open_Spread,
      "\nClose Spread:  ",Close_Spread,
      "\nMax Pips:    ",Max_Pips,
      "\nPip Count:   ", val,
      "\nNext TP:     ", nextTP, "   ", Lots2Close, " Lots",
      "\nCurr  SL:     ", sl,
      "\nTSL Trigger:   ", Trail_Trigger, "   ", trailon,
      "\nManaging:  ", mngMagic);

   }
}
//== end function getOpenOrders

//========== FUNCTION getPipValue
double getPipValue(double ord,int dir) //ord=OrderOpenPrice(),dir=OrderType(): 1=OP_SELL
{
   double val;
   RefreshRates();
   if(dir == 1)
   {
      //val=(NormalizeDouble(ord,Digits) - NormalizeDouble(Ask,Digits));
      val=NormalizeDouble(ord-Ask,Digits);
      Print("f_getPipValue: NormalizeDouble(ord,Digits)= ",NormalizeDouble(ord,Digits),".");  //debug info check
      Print("f_getPipValue: ord (OrderOpenPrice())= ",ord,".");  //debug info check
      Print("f_getPipValue: NormalizeDouble(Ask,Digits)= ",NormalizeDouble(Ask,Digits),".");  //debug info check
      Print("f_getPipValue: Ask= ",Ask,".");  //debug info check
      Print("f_getPipValue: val=(NormalizeDouble(ord,Digits) - NormalizeDouble(Ask,Digits))= ",NormalizeDouble(ord,Digits) - NormalizeDouble(Ask,Digits),".");  //debug info check
      Print("f_getPipValue: val=NormalizeDouble(ord-Ask,Digits)= ",NormalizeDouble(ord-Ask,Digits),".");  //debug info check
      Print("f_getPipValue: val=NormalizeDouble((ord-Ask)*Point,Digits)= ",NormalizeDouble((ord-Ask)*Point,Digits),".");  //debug info check
   }
   else
   {
      //val=(NormalizeDouble(Bid,Digits) - NormalizeDouble(ord,Digits));
      val=NormalizeDouble(ord-Ask,Digits);
      Print("f_getPipValue: NormalizeDouble(ord,Digits)= ",NormalizeDouble(ord,Digits),".");  //debug info check
      Print("f_getPipValue: ord (OrderOpenPrice())= ",ord,".");  //debug info check
      Print("f_getPipValue: NormalizeDouble(Bid,Digits)= ",NormalizeDouble(Bid,Digits),".");  //debug info check
      Print("f_getPipValue: Bid= ",Bid,".");  //debug info check
      Print("f_getPipValue: val=(NormalizeDouble(Bid,Digits) - NormalizeDouble(ord,Digits))= ",NormalizeDouble(Bid,Digits) - NormalizeDouble(ord,Digits),".");  //debug info check
      Print("f_getPipValue: val=NormalizeDouble(Bid-ord,Digits)= ",NormalizeDouble(Bid-ord,Digits),".");  //debug info check
      Print("f_getPipValue: val=NormalizeDouble((Bid-ord)*Point,Digits)= ",NormalizeDouble((Bid-ord)*Point,Digits),".");  //debug info check
   }
   val = val/Point;
   Print("f_getPipValue: Point= ",Point,".");  //debug info check
   Print("f_getPipValue: (val = val/Point)= ",val,".");  //debug info check
   return(val);
}
//== end function getPipValue

//========== FUNCTION moveStops
void checkStops(int pips,int ticket)
{
   Print("f_checkStops: sl= ",sl,".");  //debug info check
   Print("f_checkStops: pips= ",pips,".");  //debug info check
   if(sl==-1*Max_Loss && First_Stop != 0 && pips >= First_StopLoss_Trigger)
   {
      moveStops(ticket, First_Stop);
   }
   else if(sl==First_Stop && First_Stop != 0 && pips >= Second_StopLoss_Trigger && Second_Stop !=0)
   {
      moveStops(ticket,Second_Stop);
   }
   else if(sl==Second_Stop && Second_Stop != 0 && pips >= Third_StopLoss_Trigger && Third_Stop !=0)
   {
      moveStops(ticket,Third_Stop);
   }
   else if(Trail_Stop_By != 0 && pips >= Trail_Trigger && pips - Trail_Stop_By > sl)
   {
      Trail_Stop = pips - Trail_Stop_By;
      sl = Trail_Stop;
      trailon = " ON";  
   }
   if(pips <= sl) stopTrade(pips,OrderTicket());  
   if(pips > Max_Pips) Max_Pips = pips;
   
}
//== end function

//========== FUNCTION takeProfit
void takeProfit(int pips, int ticket)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {
      if(pips >= nextTP)
      {
         if(Alert_Sound) PlaySound("mtp2.wav");
         if(OrderType()==1)    // 1 = OP_Sell
         {
            if(OrderClose(ticket, Lots2Close, Ask, Slip, Lime)) {temp=0;
               Print("f_takeProfit: Ask= ",Ask,".");  //debug info check
               Print("f_takeProfit: MarketInfo(Symbol(),MODE_ASK)= ",MarketInfo(Symbol(),MODE_BID),".");}  //debug info check
            else ErrorDescription(GetLastError()); // Processing errors
            //Print("Error closing order : ",GetLastError());
         }
         else
         {
            if(OrderClose(ticket, Lots2Close, Bid, Slip, Lime)) {temp=0;
               Print("f_takeProfit: Bid= ",Bid,".");  //debug info check
               Print("f_takeProfit: MarketInfo(Symbol(),MODE_BID)= ",MarketInfo(Symbol(),MODE_BID),".");}  //debug info check
            else ErrorDescription(GetLastError()); // Processing errors
            //Print("Error closing order : ",GetLastError());
         }
         if(Close_Method==2) Lots2Close = Lots2Close + Close_Lots;
      
         if(nextTP == Third_TP) nextTP = nextTP + TP_Increment;
         else if (nextTP == Second_TP) nextTP = Third_TP;
         else if (nextTP == First_TP) nextTP = Second_TP;
         else nextTP = nextTP + TP_Increment;
      }
   }
}

//== end function

//========== FUNCTION moveStops
void moveStops(int ticket,int stopDiff)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {
      Print("moveStops called ",ticket, " ",stopDiff);
      if(OrderType()==1)     // 1 = OP_Sell
      {
         if (OrderModify(ticket,OrderOpenPrice(),OrderOpenPrice()-stopDiff*Point,OrderTakeProfit(),0,Plum) == true)
         {
//*            draw_horizontal_SL_line();
            Print("f_moveStops: SL of SELL order ",ticket," moved from ",sl," to ",OrderOpenPrice()-stopDiff*Point,"");//debug info check
            sl=stopDiff;
         }
         else ErrorDescription(GetLastError()); // Processing errors
      }
      else  // OrderType = 1 = OP_BUY
      {
         if (OrderModify(ticket,OrderOpenPrice(),OrderOpenPrice()+stopDiff*Point,OrderTakeProfit(),0,Plum) == true)
         {
            draw_sl_lines();
            Print("f_moveStops: SL of SELL order ",ticket," moved from ",sl," to ",OrderOpenPrice()+stopDiff*Point,"");//debug info check
            sl=stopDiff;
         }
         else ErrorDescription(GetLastError()); // Processing errors
      }
   }
}
//== end function

//========== FUNCTION killTrades
void stopTrade(int pips, int ticket)
{
   if(OrderSelect(ticket, SELECT_BY_TICKET)==true)
   {
       if(OrderType()==1) {OrderClose(ticket,OrderLots(),Ask,Slip,DarkViolet);
         Print("f_stopTrade: Ask= ",Ask,".");  //debug info check
         Print("f_stopTrade: MarketInfo(Symbol(),MODE_ASK)= ",MarketInfo(Symbol(),MODE_ASK),".");}  //debug info check
       else {OrderClose(ticket,OrderLots(),Bid,Slip,DarkViolet);
         Print("f_stopTrade: Bid= ",Bid,".");  //debug info check
         Print("f_stopTrade: MarketInfo(Symbol(),MODE_BID)= ",MarketInfo(Symbol(),MODE_BID),".");}  //debug info check
       
       if(Alert_Email) SendMail("MTP:  Trade Stopped "+pips," ");
       if(Alert_Sound) 
          if(pips > 0) PlaySound("mtp3.wav");
          else PlaySound("mtp1.wav");
          
   }
}
//== end function

//========== FUNCTION draw_horizontal_TP_line
void draw_horizontal_TP_line();
{
   if (First_TP > 0)
   {
      if (ObjectCreate("TP1", OBJ_HLINE, 0, ,TP1_price == false)  // an error occurred when drawing line
      {
         Alert("",Symbol()," Error of calling ObjectCreate():",GetLastError());
         ErrorDescription (GetLastError());
      }
      else  //no error, object was succesfuly drawn
      {
         ObjectSet("TP1",OBJPROP_COLOR,TPlinecolor);
         ObjectSet("TP1",OBJPROP_WIDTH,linewidth);
         ObjectSet("TP1",OBJPROP_STYLE,linestyle);
      }
   }
   
   if (Second_TP > 0)
   {
      if (ObjectCreate("TP2", OBJ_HLINE, 0, ,TP2_price) == false)  // an error occurred when drawing line
      {
         Alert("",Symbol()," Error of calling ObjectCreate():",GetLastError());
         ErrorDescription (GetLastError());
      }
      else
      {
         ObjectSet("TP2",OBJPROP_COLOR,TPlinecolor);
         ObjectSet("TP2",OBJPROP_WIDTH,linewidth);
         ObjectSet("TP2",OBJPROP_STYLE,linestyle);
      }
   }
   
   if (Third_TP > 0)
   {
      if (ObjectCreate("TP3", OBJ_HLINE, 0, ,TP3_price) == false)  // an error occurred when drawing line
      {
         Alert("",Symbol()," Error of calling ObjectCreate():",GetLastError());
         ErrorDescription (GetLastError());
      }
      else
      {
         ObjectSet("TP3",OBJPROP_COLOR,TPlinecolor);
         ObjectSet("TP3",OBJPROP_WIDTH,linewidth);
         ObjectSet("TP3",OBJPROP_STYLE,linestyle);
      }
   }
   
   return (0);
}
//== end function


//========== FUNCTION ErrorDescription
// This function analizes error why trade could not be opened... taken from http://book.mql4.com/samples/expert
int stderror(int Error)                        // Function of processing errors
{
   Alert("Entered on error analysis rutine");  // Alert temporal - se borrar
   switch(Error)
     {                                          // Not crucial errors            
      case  4: Alert("Trade server is busy. Trying once again..");
         Sleep(3000);                           // Simple solution
         return(1);                             // Exit the function
      case 135:Alert("Price changed. Trying once again..");
         RefreshRates();                        // Refresh rates
         return(1);                             // Exit the function
      case 136:Alert("No prices. Waiting for a new tick..");
         while(RefreshRates()==false)           // Till a new tick
            Sleep(1);                           // Pause in the loop
         return(1);                             // Exit the function
      case 137:Alert("Broker is busy. Trying once again..");
         Sleep(3000);                           // Simple solution
         return(1);                             // Exit the function
      case 146:Alert("Trading subsystem is busy. Trying once again..");
         Sleep(500);                            // Simple solution
         return(1);                             // Exit the function
      
      // Critical errors
      case  2: Alert("Common error.");
         return(0);                             // Exit the function
      case  5: Alert("Old terminal version.");
         //Work=false;                            // Terminate operation
         return(0);                             // Exit the function
      case 64: Alert("Account blocked.");
         //Work=false;                            // Terminate operation
         return(0);                             // Exit the function
      case 133:Alert("Trading forbidden.");
         return(0);                             // Exit the function
      case 134:Alert("Not enough money to execute operation.");
         return(0);                             // Exit the function
      default: Alert("Error occurred: ",Error,"");  // Other variants   
         return(0);                             // Exit the function
     }
}
//== end function